==> quarkus-narayana-jta-1.13.7.Final-sources.jar: io/quarkus/narayana/jta/runtime/interceptor/TestTransactionInterceptor.java <==
package io.quarkus.narayana.jta.runtime.interceptor;

import java.util.ArrayList;
import java.util.List;
import java.util.ServiceLoader;

import javax.inject.Inject;
import javax.interceptor.AroundInvoke;
import javax.interceptor.InvocationContext;
import javax.transaction.Status;
import javax.transaction.UserTransaction;

import io.quarkus.narayana.jta.runtime.test.TestTransactionCallback;

public class TestTransactionInterceptor {

    static final List<TestTransactionCallback> CALLBACKS;

    static {
        List<TestTransactionCallback> callbacks = new ArrayList<>();
        for (TestTransactionCallback i : ServiceLoader.load(TestTransactionCallback.class)) {
            callbacks.add(i);
        }
        CALLBACKS = callbacks;
    }

    @Inject
    UserTransaction userTransaction;

    @AroundInvoke
    public Object intercept(InvocationContext context) throws Exception {
        // do nothing in case there is already a transaction (e.g. self-intercepted non-private non-test method in test class)
        // w/o this check userTransaction.begin() would fail because there is already a tx associated with the current thread
        if (userTransaction.getStatus() != Status.STATUS_NO_TRANSACTION) {
            return context.proceed();
        }

        // an exception from proceed() has to be captured to avoid shadowing it in finally() with a exception from rollback()
        Throwable caught = null;
        try {
            userTransaction.begin();
            for (TestTransactionCallback i : CALLBACKS) {
                i.postBegin();
            }
            Object result = context.proceed();
            for (TestTransactionCallback i : CALLBACKS) {
                i.preRollback();
            }
            return result;
        } catch (Exception | Error e) { // note: "Error" shall mainly address AssertionError
            caught = e;
            throw e;
        } finally {
            if (caught == null) {
                userTransaction.rollback();
            } else {
                try {
                    userTransaction.rollback();
                } catch (Exception e) {
                    caught.addSuppressed(e);
                }
            }
        }
    }
}

==> quarkus-narayana-jta-2.10.0.Final-sources.jar: io/quarkus/narayana/jta/runtime/interceptor/TestTransactionInterceptor.java <==
package io.quarkus.narayana.jta.runtime.interceptor;

import java.util.ArrayList;
import java.util.List;
import java.util.ServiceLoader;

import javax.inject.Inject;
import javax.interceptor.AroundInvoke;
import javax.interceptor.InvocationContext;
import javax.transaction.Status;
import javax.transaction.UserTransaction;

import io.quarkus.narayana.jta.runtime.test.TestTransactionCallback;

public class TestTransactionInterceptor {

    static final List<TestTransactionCallback> CALLBACKS;

    static {
        List<TestTransactionCallback> callbacks = new ArrayList<>();
        for (TestTransactionCallback i : ServiceLoader.load(TestTransactionCallback.class)) {
            callbacks.add(i);
        }
        CALLBACKS = callbacks;
    }

    @Inject
    public UserTransaction userTransaction;

    @AroundInvoke
    public Object intercept(InvocationContext context) throws Exception {
        // do nothing in case there is already a transaction (e.g. self-intercepted non-private non-test method in test class)
        // w/o this check userTransaction.begin() would fail because there is already a tx associated with the current thread
        if (userTransaction.getStatus() != Status.STATUS_NO_TRANSACTION) {
            return context.proceed();
        }

        // an exception from proceed() has to be captured to avoid shadowing it in finally() with a exception from rollback()
        Throwable caught = null;
        try {
            userTransaction.begin();
            for (TestTransactionCallback i : CALLBACKS) {
                i.postBegin();
            }
            Object result = context.proceed();
            for (TestTransactionCallback i : CALLBACKS) {
                i.preRollback();
            }
            return result;
        } catch (Exception | Error e) { // note: "Error" shall mainly address AssertionError
            caught = e;
            throw e;
        } finally {
            if (caught == null) {
                userTransaction.rollback();
            } else {
                try {
                    userTransaction.rollback();
                } catch (Exception e) {
                    caught.addSuppressed(e);
                }
            }
        }
    }
}
